package seed.minerva.neutralBeams.aug;


import algorithmrepository.exceptions.NotImplementedException;
import oneLiners.OneLiners;
import seed.minerva.deepend.NodeImpl;
import seed.minerva.diagnostics.LineOfSightSystem;
import seed.minerva.neutralBeams.BeamDefsNode;
import seed.minerva.neutralBeams.GaussianBeamDefinition;
import seed.minerva.neutralBeams.GaussianBeamDefinitions;

/** Geometry and fuel information for the ASDEX Lithium Neutral Beams
 *  Coords from E.Wolfrum feasibility study (adjust to our '+x is North' coord system)
 *  
 *  I never actually did anything with this, but there /might/ be some data in it, so I'm leaving it here
 *  in the minervaV3 merge as a non-node. I do not know which data is from the heating beams and which is actually correct.
 *  
 *   If used, it'll need bringing in line with AugHeatingBeams structure
 */
public class AugLithiumBeam {
	public final static double electronCharge = 1.60217646e-19;
	public static final double protonMass = 1.67262158e-27; //kg
	private final static double[] deg2Rad(double deg[]){ for(int i=0; i < deg.length; i++){ deg[i] *= Math.PI / 180; } return deg; }
	private final static double[][] deg2Rad(double deg[][]){ for(int i=0; i < deg.length; i++){ deg[i] = deg2Rad(deg[i]); } return deg; }

	private double A[] = new double[]{ 0.3287917, 1.2258406, 0.3307370 };
	private double Px[] = new double[]{ 0.3555990,  2.225418, 0.3195 };
	
	
	// Some geometry is just all the same (mostly stuff we made up for now)
	/** FWHM of beam at focus (roughly, from http://dx.doi.org/10.1016/0010-4655(95)00013-6 ) */
	private static final double fwhmAtFocus = 0.015;
	
	/** Increase in FWHM per unit length along beam axis i.e. tanTheta 
	 * This is a crude estimate, based on the size of extraction grids (50.6 x 22.8 cm) at 
	 * the distance from them to the vertical (8.5m) and horizontal (6.5m) focal points.
	 * 
	 * It's also vaguely consistent with 'about 1 degree' min divergence on the NBI website. 
	 * */ 
	private static final double fwhmIncrease = 0.001; 
	
	/** FWHM of beam velocity angular spread (at any point). 
	 * FIXME: made up */ 
	private double velocityAngularVariance = 0.1*0.1 *Math.PI*Math.PI /180/180;

	/** Spread of each component as fraction of calculated energy
			(i.e the 1/2 component is expected to have 1/2 the deviation of the full) 
			FIXME: made up */	
	public double energyFractionalStdDeviation = 0.015;  
		
	/** Atomic No and mass per fuel type [BEAMFUEL_*] */
	private static final int atomicNo = 3;
	private static final int atomicMass = 7; //6.941; FIXME: Err, oops, I forget these arn't actually integers
	
	/** Beam accelerating voltage in V [BEAMBOX_*][BEAMFUEL_*] */
	private static final double accelVoltage = 30e3; //40-80keV range, normal is 40keV 

	/** Perveance at optimum in AV^(-3/2) [BEAMBOX_*][BEAMFUEL_*]  */
	private static final double beamCurrent = 2.4e-3;
	
	private static final double transmission = 1.0; //FIXME: Err...
	
	private GaussianBeamDefinition beamDef;
	
	public void calcBeams() {
		
		beamDef = new GaussianBeamDefinition();
			
		//first sort out the geometry
		double u[] = new double[3];
	
		u[0] = (A[0] - Px[0]);
		u[1] = (A[1] - Px[1]);
		u[2] = (A[2] - Px[2]);
		
		double l = Math.sqrt(u[0]*u[0] + u[1]*u[1] + u[2]*u[2]);
		u[0] /= l; u[1] /= l; u[2] /= l;
		
		beamDef.uVec = u;
		
		//FIXME: Where actually is the focus? Here we assume it conincides with the beam crossing focus
		beamDef.focus = Px; 
		
		beamDef.fwhmAtFocus = fwhmAtFocus;
		beamDef.fwhmIncrease = fwhmIncrease;
		beamDef.velocityAngularVariance = velocityAngularVariance;
		
		
		//and now the fuel/accel info		
		beamDef.atomicNo = atomicNo;
		beamDef.atomicMass = atomicMass;

		//do we only have 1 component?
		beamDef.cmpPowerFrac = new double[]{ 1.0 };
		beamDef.cmpNumberFrac = new double[]{ 1.0 };
		
		beamDef.cmpEnergyMean = new double[]{ accelVoltage };
		beamDef.cmpEnergyVar = new double[]{ (energyFractionalStdDeviation * beamDef.cmpEnergyMean[0]) 
										* (energyFractionalStdDeviation * beamDef.cmpEnergyMean[0]) };
		
		//no idea how charged they were, presumably not totally stripped
		//so we assume only 1+ for now
		beamDef.cmpInjectionRate = new double[]{ beamCurrent * transmission / (/*atomicNo*/1 * electronCharge) };

	}
		
}
